package jp.savethememory.manage.util;

//
// インポート
//
import java.util.*;
import org.w3c.dom.*;

/**
 *Node内の指定したタグ名のElementNodeに対するイテレータクラスです。<BR>
 *ただし、remove()メソッドはサポートしません。
 * @since V1.0
 */
public class SpecifiedElementIterator implements Iterator<Element>{

	/**
	 *Iteratorのルートノード。<BR>
	 * @since  V1.0
	 */
	private Node topNode;

	/**
	 *現在のノードのポジション。<BR>
	 * @since  V1.0
	 */
	private int currentPosition;

	/**
	 *内部的に使用するNodeList。<BR>
	 * @since  V1.0
	 */
	private NodeList nList;

	/**
	 *nextで返されるElement。<BR>
	 * @since  V1.0
	 */
	private Element nextElement;

	/**
	 *指定されたタグ名<BR>
	 * @since V1.0
	 */
	private String tagName;

	/**
	 *SpecifiedElementIteratorクラスを構築します。<BR>
	 * @since V1.0
	 */
	private SpecifiedElementIterator(){
		currentPosition = -1;
	}

	/**
	 *SpecifiedElementIteratorクラスを構築します。<BR>
	 * @param node イテレータの対象となるNode
	 * @param tag 指定するTag名
	 * @since V1.0
	 */
	public SpecifiedElementIterator(Node node, String tag){
		this();
		topNode = node;
		tagName = tag;
		nList = topNode.getChildNodes();
		nextElement = getNextElement();
	}

	/**
	 *繰り返し処理でさらに要素がある場合に true を返します。<BR>
	 *つまり、next が例外をスローしないで要素を返す場合に true を返します。
	 * @return 反復子がさらに要素を持つ場合は true
	 * @since V1.0
	 */
	public boolean hasNext(){
		return nextElement != null;
	}

	/**
	 *繰り返し処理で次の要素を返します。<BR>
	 * @return 繰り返し処理で次の要素
	 * @since V1.0
	 */
	public Element next(){
		if (nextElement == null){
			throw new NoSuchElementException("このNodeにはこれ以上Elementがありません。");
		}

		Element elm = nextElement;
		nextElement = getNextElement();
		return elm;
	}

	/**
	 *基になるコレクションから、反復子によって最後に返された要素を削除します (任意のオペレーション)。<BR>
	 *SpecifiedElementIteratorクラスでは未サポートです。
	 * @since V1.0
	 */
	public void remove(){
		throw new UnsupportedOperationException("このメソッドはサポート外です。");
	}

	/**
	 *次に返す予定のElementを取得します。
	 * @return 次に返すElement ない場合はnull
	 * @since V1.0
	 */
	private Element getNextElement(){
		Node hitNode = null;
		for (int i = currentPosition + 1; i < nList.getLength() ; i++){
			Node node = nList.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE){
				if (node.getNodeName().equals(tagName) == true){
					hitNode = node;
					currentPosition = i;
					break;
				}
			}
		}
		return (Element)hitNode;
	}
}
///////////// end of SpecifiedElementIterator.java /////////////
