package jp.savethememory.manage.util;

//
// インポート
//
import org.w3c.dom.*;

import java.util.*;

/**
 *XMLに関するユーティリティクラスです。<BR>
 * @since V1.0
 */
public class XMLUtilities{
	/**
	 *階層捜査用Nodeセパレータ。<BR>
	 * @since V1.0
	 */
	private static final String NODE_SEPARATOR = "/";
	/**
	 *階層捜査用TextNodeセパレータ。<BR>
	 * @since V1.0
	 */
	private static final String TEXTNODE_SEPARATOR = "/@";

	/**
	 *このクラスを外部でnewさせないための制御コンストラクタです。<BR>
	 * @since V1.0
	 */
	private XMLUtilities(){}

	/**
	 *指定されたNodeに指定されたタグ名でデータを挿入します。<BR>
	 * @param node データを入れるNode
	 * @param tag データを入れるタグ名
	 * @param value データ
	 * @see #setData(Document, Node, String, String)
	 * @since V1.0
	 */
	static public void setData(Node node, String tag, String value){
		XMLUtilities.setData(node.getOwnerDocument(), node, tag, value);
	}

	/**
	 *指定されたNodeに指定されたタグ名でデータを挿入します。<BR>
	 *たとえば以下のようなノードを第２引数に指定した場合の例を示します。<BR>
	 *<P>
	 *<UL>
	 *&lt;target&gt;&lt;first&gt;&lt;/first&gt;&lt;second&gt;2&lt;/second&gt;&lt;/target&gt;
	 *</UL><BR>
	 *１、tag = "first", value = "いち"を指定した場合の結果
	 *<UL>
	 *&lt;target&gt;&lt;first&gt;いち&lt;/first&gt;&lt;second&gt;2&lt;/second&gt;&lt;/target&gt;
	 *</UL><BR>
	 *２、tag = "second", value = "に"を指定した場合の結果
	 *<UL>
	 *&lt;target&gt;&lt;first&gt;&lt;/first&gt;&lt;second&gt;に&lt;/second&gt;&lt;/target&gt;
	 *</UL><BR>
	 *３、tag = "third", value = "さん"を指定した場合の結果
	 *<UL>
	 *&lt;target&gt;&lt;first&gt;&lt;/first&gt;&lt;second&gt;2&lt;/second&gt;&lt;third&gt;さん&lt;/third&gt;&lt;/target&gt;
	 *</UL>
	 *</P>
	 * @param doc 追加するNodeのオーナDocument
	 * @param node データを入れるNode
	 * @param tag データを入れるタグ名
	 * @param value データ
	 * @since V1.0
	 */
	static public void setData(Document doc, Node node, String tag, String value){
		NodeList nodeList = node.getChildNodes();
		Node target = null;
		int nodeCount = nodeList.getLength();
		for (int i = 0 ; i < nodeCount ; i++){
			target = nodeList.item(i);
			if (target.getNodeName().equals(tag) == true){
				break;
			}
			target = null;
		}
		if (target == null){
			// 指定されたNodeにターゲットのタグ名が存在しない場合。
			target = doc.createElement(tag);
			node.appendChild(target);
		} else {
			// 一度、ターゲットの子ノードを削除します。
			XMLUtilities.clearNode(target);
		}
		Text text = doc.createTextNode(value);
		target.appendChild(text);
	}

	/**
	 *指定されたNodeに指定されたタグ名でデータを挿入します。<BR>
	 * @param node データを入れるNode
	 * @param tag データを入れるタグ名
	 * @param value データ
	 * @see #setData(Document, Node, String, String)
	 * @since V1.0
	 */
	static public void setData(Node node, String tag, int value){
		XMLUtilities.setData(node.getOwnerDocument(), node, tag, value);
	}

	/**
	 *指定されたNodeに指定されたタグ名でデータを挿入します。<BR>
	 * @param doc 追加するNodeのオーナDocument
	 * @param node データを入れるNode
	 * @param tag データを入れるタグ名
	 * @param value データ
	 * @see #setData(Document, Node, String, String)
	 * @since V1.0
	 */
	static public void setData(Document doc, Node node, String tag, int value){
		XMLUtilities.setData(doc, node, tag, Integer.toString(value));
	}

	/**
	 *指定されたノードにノードとデータを追加します。<BR>
	 *たとえば以下のようなノードを第２引数に指定した場合の例を示します。<BR>
	 *<P>
	 *<UL>
	 *&lt;target&gt;&lt;first&gt;&lt;/first&gt;&lt;/target&gt;
	 *</UL><BR>
	 *１、tag = "first", value = "いち"を指定した場合の結果
	 *<UL>
	 *&lt;target&gt;&lt;first&gt;&lt;/first&gt;&lt;first&gt;いち&lt;/first&gt;&lt;/target&gt;
	 *</UL><BR>
	 *２、tag = "second", value = "に"を指定した場合の結果
	 *<UL>
	 *&lt;target&gt;&lt;first&gt;&lt;/first&gt;&lt;second&gt;に&lt;/second&gt;&lt;/target&gt;
	 *</UL><BR>
	 *</P>
	 * @param doc 追加するNodeのオーナドキュメント
	 * @param node データを追加するNode
	 * @param tag データを入れるタグ名
	 * @param value データ
	 * @return 追加したNode
	 * @since V1.0
	 */
	static public Node appendData(Document doc, Node node, String tag, String value){
		Node target = doc.createElement(tag);
		Node text = doc.createTextNode(value);
		target.appendChild(text);
		node.appendChild(target);
		return target;
	}

	/**
	 *指定されたノードにノードを追加します。<BR>
	 *たとえば以下のようなノードを第２引数に指定した場合の例を示します。<BR>
	 *<P>
	 *<UL>
	 *&lt;target&gt;&lt;first&gt;&lt;/first&gt;&lt;/target&gt;
	 *</UL><BR>
	 *１、tag = "first"を指定した場合の結果
	 *<UL>
	 *&lt;target&gt;&lt;first&gt;&lt;/first&gt;&lt;first&gt;&lt;/first&gt;&lt;/target&gt;
	 *</UL><BR>
	 *２、tag = "second"を指定した場合の結果
	 *<UL>
	 *&lt;target&gt;&lt;first&gt;&lt;/first&gt;&lt;second&gt;&lt;/second&gt;&lt;/target&gt;
	 *</UL><BR>
	 *</P>
	 * @param doc 追加するNodeのオーナドキュメント
	 * @param node データを追加するNode
	 * @param tag データを入れるタグ名
	 * @return 追加したNode
	 * @since V1.0
	 */
	static public Node appendNode(Document doc, Node node, String tag){
		Node target = doc.createElement(tag);
		node.appendChild(target);
		return target;
	}

	/**
	 *指定されたノードにノードを追加します。<BR>
	 * @param node データを追加するNode
	 * @param tag データを入れるタグ名
	 * @return 追加したNode
	 * @see #appendNode(Document, Node, String)
	 * @since V1.0
	 */
	static public Node appendNode(Node node, String tag){
		return XMLUtilities.appendNode(node.getOwnerDocument(), node, tag);
	}


	/**
	 *指定されたノード以下のノードにテキストを追加します。<BR>
	 *たとえば以下のようなノードを第２引数に指定した場合の例を示します。<BR>
	 *<P>
	 *<UL>
	 *&lt;target&gt;&lt;first&gt;&lt;/first&gt;&lt;second&gt;2&lt;/second&gt;&lt;/target&gt;
	 *</UL><BR>
	 *１、tag = "first", value = "いち"を指定した場合の結果
	 *<UL>
	 *&lt;target&gt;&lt;first&gt;いち&lt;/first&gt;&lt;second&gt;2&lt;/second&gt;&lt;/target&gt;
	 *</UL><BR>
	 *２、tag = "second", value = "に"を指定した場合の結果
	 *<UL>
	 *&lt;target&gt;&lt;first&gt;&lt;/first&gt;&lt;second&gt;2に&lt;/second&gt;&lt;/target&gt;
	 *</UL><BR>
	 *３、tag = "third", value = "さん"を指定した場合の結果
	 *<UL>
	 *&lt;target&gt;&lt;first&gt;&lt;/first&gt;&lt;second&gt;2&lt;/second&gt;&lt;third&gt;さん&lt;/third&gt;&lt;/target&gt;
	 *</UL><BR>
	 *</P>
	 * @param doc 追加するNodeのオーナドキュメント
	 * @param node データを追加するNode
	 * @param tag データを入れるタグ名
	 * @param value データ
	 * @since V1.0
	 */
	static public void appendText(Document doc, Node node, String tag, String value){
		Node target = XMLUtilities.getNode(node, tag);
		if (target == null){
			target = XMLUtilities.appendNode(doc, node, tag);
		}
		Text text = doc.createTextNode(value);
		target.appendChild(text);
	}

	/**
	 *Node内にある指定されたタグ名のNodeを返します。<BR>
	 * @param node 指定したタグのあるNode
	 * @param tag 指定するタグ名
	 * @return 指定したタグ名のNode,存在しない場合はnullを返す。
	 * @since V1.0
	 */
	static public Node getNode(Node node, String tag){
		NodeList nodeList = node.getChildNodes();
		Node targetNode = null;
		for (int i = 0 ; i < nodeList.getLength() ; i++){
			Node currentNode = nodeList.item(i);
			if (currentNode.getNodeName().equals(tag) == true){
				targetNode = currentNode;
				break;
			}
		}
		return targetNode;
	}



	/**
	 *指定されたNodeから相対位置で指定して対象となるNodeを返します。<BR>
	 *引数keyの指定方法は{@link #getNodes(Node, String)}と同じです。ただし、getNodesメソッドとは違い、最初の１つしか返しません。
	 * @param node 対象となるNode
	 * @param key キー
	 * @return 取得したNode
	 * @since V1.0
	 */
	static public Node getRelativeNode(Node node, String key){
		StringTokenizer keyTokenizer = new StringTokenizer(key, NODE_SEPARATOR);

		Node targetNode = node;
		while(keyTokenizer.hasMoreTokens()){
			String str = keyTokenizer.nextToken();
			targetNode = XMLUtilities.getNode(targetNode, str);
			if (targetNode == null){
				break;
			}
		}
		if (targetNode == node){
			targetNode = null;
		}
		return targetNode;
	}

	/**
	 *指定されたNodeから相対位置を指定して対象となるNodeのリストを返します。<BR>
	 *keyはたとえば以下のような形式で指定します。<BR>
	 *<UL>
	 *<LI>key1/key2/key3<BR>
	 *</UL>
	 *このように指定すると、指定されたNode以下にあるタグ名が"key1"というノードを全て検索し、それ以下のNodeから
	 *タグ名が"key2"であるNodeを全て検索します。さらにそれ以下のNodeからタグ名が"key3"であるノードを全て検索し、
	 *そのNodeを配列として返します。
	 * @param node 対象となるNode
	 * @param key キー
	 * @return 取得したNode
	 * @since V1.0
	 * @throw IllegalArgumentException キーの指定が不正な場合
	 */
	static public Node[] getNodes(Node node, String key){
		StringTokenizer keyTokenizer = new StringTokenizer(key, NODE_SEPARATOR);
		List<Node> targetList = new ArrayList<Node>();
		targetList.add(node);
		while(keyTokenizer.hasMoreTokens()){
			String str = keyTokenizer.nextToken();
			List<Node> getList = new ArrayList<Node>();
			for (int i = 0 ; i < targetList.size() ; i++){
				SpecifiedElementIterator sei = new SpecifiedElementIterator((Node)targetList.get(i), str);
				while(sei.hasNext()){
					getList.add(sei.next());
				}
			}
			targetList = getList;
		}

		Node[] nodes = new Node[0];
		nodes = targetList.toArray(nodes);
		return nodes;
	}

	/**
	 *指定されたNodeから相対位置を指定して対象となるNodeにあるテキストのリストを返します。<BR>
	 *keyはたとえば以下のような形式で指定します。
	 *<UL>
	 *<LI>key1/key2/@key3<BR>
	 *</UL>
	 *取得対象となるNodeは{@link #getNodes(Node, String)}と同様です。
	 *このメソッドと指定すると、最下層のNode以下にあるテキストノード内の文字列を取得します。
	 * @param node 対象となるNode
	 * @param key キー
	 * @return 取得した文字列
	 * @since V1.0
	 * @throw IllegalArgumentException キーの指定が不正な場合
	 */
	static public String[] getTexts(Node node, String key){
		int index = key.lastIndexOf(TEXTNODE_SEPARATOR);

		if (index < 0){
			throw new IllegalArgumentException("keyの指定方法が不正です。");
		}

		String nodeKey = key.substring(0, index + 1) + key.substring(index + 2);
		Node[] nodes = XMLUtilities.getNodes(node, nodeKey);
		String[] texts = new String[nodes.length];

		for (int i = 0 ; i < nodes.length ; i++){
			texts[i] = XMLUtilities.getText(nodes[i]);
		}

		return texts;
	}

	/**
	 *Nodeの直下にある最初のTextNodeのデータを取得します。
	 * @param node 指定するNode
	 * @return テキスト,TextNodeが存在しない場合はnullを返す。
	 * @since V1.0
	 */
	static public String getText(Node node){
		NodeList nodeList = node.getChildNodes();
		String text = null;
		for (int i = 0 ; i < nodeList.getLength() ; i++){
			Node target = nodeList.item(i);
			if (target.getNodeType() == Node.TEXT_NODE){
				text = target.getNodeValue();
			}
		}
		return text;
	}

	/**
	 *ElementNodeを生成します。<BR>
	 * @param node 元となるNode
	 * @param tag 生成するElementNodeのタグ名
	 * @return 生成したElementNode
	 * @since V1.0
	 */
	static public Element createElementNode(Node node, String tag){
		Document doc = node.getOwnerDocument();
		Element element = doc.createElement(tag);
		return element;
	}

	/**
	 *TextNodeを生成します。<BR>
	 * @param node 元となるNode
	 * @param text 生成するTextNodeのデータ
	 * @return 生成したTextNode
	 * @since V1.0
	 */
	static public Text createTextNode(Node node, String text){
		Document doc = node.getOwnerDocument();
		Text textNode = doc.createTextNode(text);
		return textNode;
	}

	/**
	 *指定されたデータのTextNodeの入った指定されたタグ名のElementを作成し、指定された親Nodeに指定されたNodeの前に挿入する。
	 *たとえば以下のようなノードを第２引数に指定した場合の例を示します。<BR>
	 *<P>
	 *<UL>
	 *&lt;target&gt;&lt;first&gt;&lt;/first&gt;&lt;second&gt;&lt;/second&gt;&lt;/target&gt;
	 *</UL><BR>
	 *１、refNode = first, tag = "newData", value = "追加"を指定した場合の結果
	 *<UL>
	 *&lt;target&gt;&lt;newData&gt;追加&lt;/newData&gt;&lt;first&gt;いち&lt;/first&gt;&lt;second&gt;2&lt;/second&gt;&lt;/target&gt;
	 *</UL><BR>
	 *２、refNode = second, tag = "newData", value = "追加"を指定した場合の結果
	 *<UL>
	 *&lt;target&gt;&lt;first&gt;&lt;/first&gt;&lt;newData&gt;追加&lt;/newData&gt;&lt;second&gt;に&lt;/second&gt;&lt;/target&gt;
	 *</UL><BR>
	 *</P>
	 * @param doc オーナDocument
	 * @param parentNode 親Node
	 * @param refNode 次にあたるNode
	 * @param tag タグ名
	 * @param text データ
	 * @return 追加したElement
	 * @since V1.0
	 */
	static public Element insertElement(Document doc, Node parentNode, Node refNode, String tag, String text){
		Element newNode = doc.createElement(tag);
		Text textNode = doc.createTextNode(text);
		newNode.appendChild(textNode);
		parentNode.insertBefore(newNode, refNode);
		return newNode;
	}

	/**
	 *指定されたデータのTextNodeの入った指定されたタグ名のElementを作成し、指定された親Nodeに指定されたNodeの前に挿入する。
	 * @param parentNode 親Node
	 * @param refNode 次にあたるNode
	 * @param tag タグ名
	 * @param text データ
	 * @return 追加したElement
	 * @see #insertElement(Document, Node, Node, String, String)
	 * @since V1.0
	 */
	static public Element insertElement(Node parentNode, Node refNode, String tag, String text){
		return insertElement(refNode.getOwnerDocument(), parentNode, refNode, tag, text);
	}

	/**
	 *指定されたNodeのChildNodeをすべて削除します。<BR>
	 * @param node クリアするNode
	 * @since V1.0
	 */
	static public void clearNode(Node node){
		NodeList nodeList = node.getChildNodes();
		int count = nodeList.getLength();
		for (int i = 0 ; i < count ; i++){
			node.removeChild(nodeList.item(0));
		}
	}

	/**
	 *XMLのドキュメントを指定されたファイルに保存します。<BR>
	 * @param document XMLドキュメント
	 * @param filename 保存するファイル名
	 * @throws FileNotFoundException
	 * @throws IOException
	 * @since V1.0
	 */
//	static public void writeDocument(Document document, String filename) throws FileNotFoundException, IOException{
//		OutputFormat format = new OutputFormat();
//
//		format.setEncoding("Shift_JIS");
//
//		FileOutputStream os = new FileOutputStream(filename);
//
//		XMLSerializer serial = new XMLSerializer(os,format);
//		DOMSerializer ds = serial.asDOMSerializer();
//		ds.serialize(document);
//		os.flush();
//		os.close();
//	}
}
///////////// end of XMLUtilities.java /////////////
